# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formGroupBox = QtWidgets.QGroupBox(Form)
        self.formGroupBox.setObjectName("formGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.formGroupBox)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.alphaLabel = QtWidgets.QLabel(self.formGroupBox)
        self.alphaLabel.setObjectName("alphaLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.alphaLabel)
        self.alphaDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.formGroupBox)
        self.alphaDoubleSpinBox.setObjectName("alphaDoubleSpinBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.alphaDoubleSpinBox)
        self.fit_interceptLabel = QtWidgets.QLabel(self.formGroupBox)
        self.fit_interceptLabel.setObjectName("fit_interceptLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.fit_interceptLabel)
        self.fit_interceptCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.fit_interceptCheckBox.setObjectName("fit_interceptCheckBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.fit_interceptCheckBox)
        self.verboseLabel = QtWidgets.QLabel(self.formGroupBox)
        self.verboseLabel.setObjectName("verboseLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.verboseLabel)
        self.verboseCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.verboseCheckBox.setObjectName("verboseCheckBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.verboseCheckBox)
        self.normalizeLabel = QtWidgets.QLabel(self.formGroupBox)
        self.normalizeLabel.setObjectName("normalizeLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.normalizeLabel)
        self.normalizeCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.normalizeCheckBox.setObjectName("normalizeCheckBox")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.normalizeCheckBox)
        self.precomputeLabel = QtWidgets.QLabel(self.formGroupBox)
        self.precomputeLabel.setObjectName("precomputeLabel")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.precomputeLabel)
        self.precomputeComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.precomputeComboBox.setObjectName("precomputeComboBox")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.precomputeComboBox)
        self.max_iterLabel = QtWidgets.QLabel(self.formGroupBox)
        self.max_iterLabel.setObjectName("max_iterLabel")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.max_iterLabel)
        self.max_iterSpinBox = QtWidgets.QSpinBox(self.formGroupBox)
        self.max_iterSpinBox.setObjectName("max_iterSpinBox")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.max_iterSpinBox)
        self.copy_XLabel = QtWidgets.QLabel(self.formGroupBox)
        self.copy_XLabel.setObjectName("copy_XLabel")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.copy_XLabel)
        self.copy_XCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.copy_XCheckBox.setObjectName("copy_XCheckBox")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.copy_XCheckBox)
        self.fit_pathLabel = QtWidgets.QLabel(self.formGroupBox)
        self.fit_pathLabel.setObjectName("fit_pathLabel")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.fit_pathLabel)
        self.fit_pathCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.fit_pathCheckBox.setObjectName("fit_pathCheckBox")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.fit_pathCheckBox)
        self.positiveLabel = QtWidgets.QLabel(self.formGroupBox)
        self.positiveLabel.setObjectName("positiveLabel")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.positiveLabel)
        self.positiveCheckBox = QtWidgets.QCheckBox(self.formGroupBox)
        self.positiveCheckBox.setObjectName("positiveCheckBox")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.positiveCheckBox)
        self.cvLabel = QtWidgets.QLabel(self.formGroupBox)
        self.cvLabel.setObjectName("cvLabel")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.LabelRole, self.cvLabel)
        self.cvSpinBox = QtWidgets.QSpinBox(self.formGroupBox)
        self.cvSpinBox.setObjectName("cvSpinBox")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.FieldRole, self.cvSpinBox)
        self.max_n_alphasLabel = QtWidgets.QLabel(self.formGroupBox)
        self.max_n_alphasLabel.setObjectName("max_n_alphasLabel")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.LabelRole, self.max_n_alphasLabel)
        self.max_n_alphasSpinBox = QtWidgets.QSpinBox(self.formGroupBox)
        self.max_n_alphasSpinBox.setObjectName("max_n_alphasSpinBox")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.FieldRole, self.max_n_alphasSpinBox)
        self.n_jobsLabel = QtWidgets.QLabel(self.formGroupBox)
        self.n_jobsLabel.setObjectName("n_jobsLabel")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.LabelRole, self.n_jobsLabel)
        self.n_jobsSpinBox = QtWidgets.QSpinBox(self.formGroupBox)
        self.n_jobsSpinBox.setObjectName("n_jobsSpinBox")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.FieldRole, self.n_jobsSpinBox)
        self.criterionLabel = QtWidgets.QLabel(self.formGroupBox)
        self.criterionLabel.setObjectName("criterionLabel")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.LabelRole, self.criterionLabel)
        self.criterionComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.criterionComboBox.setObjectName("criterionComboBox")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.FieldRole, self.criterionComboBox)
        self.modelLabel = QtWidgets.QLabel(self.formGroupBox)
        self.modelLabel.setObjectName("modelLabel")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.LabelRole, self.modelLabel)
        self.modelComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.modelComboBox.setObjectName("modelComboBox")
        self.modelComboBox.addItem("")
        self.modelComboBox.addItem("")
        self.modelComboBox.addItem("")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.FieldRole, self.modelComboBox)
        self.verticalLayout.addWidget(self.formGroupBox)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.alphaLabel.setText(("alpha"))
        self.alphaDoubleSpinBox.setToolTip(_translate("Form", "Constant that multiplies the penalty term. Defaults to 1.0. alpha = 0\n"
"is equivalent to an ordinary least square, solved by LinearRegression.\n"
"For numerical reasons, using alpha = 0 with the LassoLars object is not\n"
"advised and you should prefer the LinearRegression object."))
        self.alphaDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.fit_interceptLabel.setText(("fit_intercept"))
        self.fit_interceptCheckBox.setToolTip(_translate("Form", "whether to calculate the intercept for this model. If set to false,\n"
"no intercept will be used in calculations (e.g. data is expected to\n"
"be already centered)."))
        self.fit_interceptCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.verboseLabel.setText(("verbose"))
        self.verboseCheckBox.setToolTip(("Sets the verbosity amount"))
        self.verboseCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.normalizeLabel.setText(("normalize"))
        self.normalizeCheckBox.setToolTip(_translate("Form", "This parameter is ignored when fit_intercept is set to False. If True,\n"
"the regressors X will be normalized before regression by subtracting\n"
"the mean and dividing by the l2-norm. If you wish to standardize,\n"
"please use sklearn.preprocessing.StandardScaler before calling fit\n"
"on an estimator with normalize=False."))
        self.normalizeCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.precomputeLabel.setText(("precompute"))
        self.precomputeComboBox.setToolTip(_translate("Form", "Whether to use a precomputed Gram matrix to speed up calculations. If\n"
"set to \'auto\' let us decide. The Gram matrix can also be passed as\n"
"argument."))
        self.precomputeComboBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.max_iterLabel.setText(_translate("Form", "max_iter\n"
""))
        self.max_iterSpinBox.setToolTip(("Maximum number of iterations to perform."))
        self.max_iterSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.copy_XLabel.setText(_translate("Form", "copy_X\n"
""))
        self.copy_XCheckBox.setToolTip(("If True, X will be copied; else, it may be overwritten."))
        self.copy_XCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.fit_pathLabel.setText(_translate("Form", "fit_path\n"
""))
        self.fit_pathCheckBox.setToolTip(_translate("Form", "If True the full path is stored in the coef_path_ attribute. If you\n"
"compute the solution for a large problem or many targets, setting\n"
"fit_path to False will lead to a speedup, especially with a small\n"
"alpha."))
        self.fit_pathCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.positiveLabel.setText(("positive"))
        self.positiveCheckBox.setToolTip(_translate("Form", "Restrict coefficients to be >= 0. Be aware that you might want to\n"
"remove fit_intercept which is set True by default. Under the positive\n"
"restriction the model coefficients will not converge to the\n"
"ordinary-least-squares solution for small values of alpha. Only\n"
"coefficients up to the smallest alpha value\n"
"(alphas_[alphas_ > 0.].min() when fit_path=True) reached by the\n"
"stepwise Lars-Lasso algorithm are typically in congruence with the\n"
"solution of the coordinate descent Lasso estimator."))
        self.positiveCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.cvLabel.setText(_translate("Form", "cv\n"
""))
        self.cvSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.max_n_alphasLabel.setText(_translate("Form", "max_n_alphas\n"
""))
        self.max_n_alphasSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.n_jobsLabel.setText(("n_jobs"))
        self.n_jobsSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.criterionLabel.setText(("criterion "))
        self.criterionComboBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.modelLabel.setText(("Model"))
        self.modelComboBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoLars.html"))
        self.modelComboBox.setItemText(0, ("Lasso Lars"))
        self.modelComboBox.setItemText(1, ("Cross Validated Lasso Lars"))
        self.modelComboBox.setItemText(2, ("Information Criterion Lasso Lars"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




import multiprocessing as mp
import os.path
import pickle
import sys
import time
import warnings

from point_spectra_gui.util.themes import braceyourself, default

try:
    import qtmodern.styles

    q = True
except:
    q = False
    warnings.warn("You're missing the qtmodern package."
                  "to install it use pip install qtmodern")
import os
import logging
import platform
import traceback
from time import strftime

from PyQt5.QtCore import *
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import *
from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import QSettings

from point_spectra_gui import core, __version__
from point_spectra_gui.ui.MainWindow import Ui_MainWindow
from point_spectra_gui.util import delete
from point_spectra_gui.util.BasicFunctionality import Basics
from point_spectra_gui.util.excepthook import my_exception_hook


class EmittingStream(QtCore.QObject):
    textWritten = QtCore.pyqtSignal(str)

    def write(self, text):
        self.textWritten.emit(str(text))

    def flush(self):
        pass


class TitleWindow:
    def __init__(self, mainName):
        self.mainName = mainName
        self.fileName = ''
        self.debugName = ''

    def setMainName(self, name):
        self.mainName = name

    def setFileName(self, name):
        self.fileName = name

    def setDebugName(self, bool):
        if bool:
            self.debugName = "Debug Mode"
        else:
            self.debugName = ''

    def display(self):
        if self.fileName == '' and self.debugName == '':
            return "{}".format(self.mainName)
        elif self.fileName == '':
            return "{} - {}".format(self.mainName, self.debugName)
        elif self.debugName == '':
            return "{} - {}".format(self.mainName, self.fileName)
        else:
            return "{} - {} - {}".format(self.mainName, self.fileName, self.debugName)


class MainWindow(Ui_MainWindow, QtCore.QThread, Basics):
    taskFinished = QtCore.pyqtSignal()

    def __init__(self):
        super().__init__()
        self.widgetList = []
        self.leftOff = 0

    def setupUi(self, MainWindow):
        super().setupUi(MainWindow)  # Run the basic window UI
        self.MainWindow = MainWindow
        self.MainWindow.closeEvent = self.closeEvent
        self.title = TitleWindow(self.MainWindow.windowTitle())
        self._readAndApplyWindowAttributeSettings()
        self.menu_item_shortcuts()  # set up the shortcuts
        self.connectWidgets()

        # Check the mode for debugging
        if self.settings.value("debug") == 'true':
            self.debug_mode()
        else:
            self.normal_mode()

        # Check the theme for the UI
        if self.settings.value('theme') == 'braceyourself':
            braceyourself(self.MainWindow)

    def normal_mode(self):
        """
        Change the direction of stdout to print to the UI console instead

        :return:
        """
        sys.stdout = EmittingStream(textWritten=self.normalOutputWritten)
        sys.stderr = sys.__stderr__
        # sys.stderr = EmittingStream(textWritten=self.normalOutputWritten)
        self._mode(self.actionOn, self.actionOff, False)

    def debug_mode(self):
        """
        Change the direction of stdout to print to the original console

        :return:
        """
        sys.stdout = EmittingStream(textWritten=self.normalOutputWritten)
        sys.stderr = sys.__stderr__
        self._mode(self.actionOff, self.actionOn, True)

    def _mode(self, obj1, obj2, debug):
        """
        Set the UI in debug or non-debug mode
        Save all the settings and grey out the necessary boxes

        :param obj1:
        :param obj2:
        :param debug:
        :return:
        """
        obj1.setDisabled(False)
        obj2.setDisabled(True)
        self.debug = debug
        self.title.setDebugName(debug)
        self.settings.setValue("debug", self.debug)
        self.MainWindow.setWindowTitle(self.title.display())

    def theme(self, name):
        """
        We have 3 themes
        each has a different situation
                   __________________
                 /__               __\ new()   Something to note:
               v    \            v    \ new()  As you can see whenever moving into or out of
        default     braceyourself    qtmodern  qtmodern we will have to start a new session
               \___^             \___^ new()   default and braceyourself can simply change
                \___________________/ new()    there styling on the spot

        :param name:
        :return:
        """
        settings = self.settings.value('theme')
        if name == settings:
            print("This is already your current theme")

        elif name == 'qtmodern':  # User is entering into qtmodern
            self.settings.setValue('theme', name)
            self.new()

        elif settings == 'qtmodern':  # User is leaving qtmodern
            self.settings.setValue('theme', name)
            self.new()

        elif name == 'default':
            self.settings.setValue('theme', name)
            default(self.MainWindow)

        elif name == 'braceyourself':
            self.settings.setValue('theme', name)
            braceyourself(self.MainWindow)
        else:
            print("Something went horribly wrong with your theme, try again?")

    def normalOutputWritten(self, text):
        """Append text to the QTextEdit."""
        # Maybe QTextEdit.append() works as well, but this is how I do it:
        cursor = self.textBrowser.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertText(text)
        self.textBrowser.setTextCursor(cursor)
        self.textBrowser.ensureCursorVisible()

    def addWidget(self, obj):
        """
        Organize our widgets using a list
        Each widget is addressed separately due to being in a list

        :param obj:
        :return:
        """
        self.widgetList.append(obj())
        self.widgetList[-1].setupUi(self.centralwidget)
        self.widgetLayout = QtWidgets.QVBoxLayout()
        self.widgetLayout.setObjectName("widgetLayout")
        self.verticalLayout_3.addLayout(self.widgetLayout)
        self.widgetLayout.addWidget(self.widgetList[-1].get_widget())
        # this should scroll the view all the way down after adding the new widget.
        scrollbar = self.scrollArea.verticalScrollBar()
        # this should scroll the view all the way down after adding the new widget.
        scrollbar.setValue(scrollbar.maximum())
        pass

    def menu_item_shortcuts(self):
        self.actionExit.setShortcut("ctrl+Q")
        self.actionClear_Workflow.setShortcut("ctrl+K")
        self.actionCreate_New_Workflow.setShortcut("ctrl+N")
        self.actionRestore_Workflow.setShortcut("ctrl+O")
        self.actionSave_Current_Workflow.setShortcut("ctrl+S")
        self.okPushButton.setShortcut("Ctrl+Return")

    def connectWidgets(self):
        """
        Connect all the widgets associated with the MainWindow UI

        :return:
        """

        try:
            self.actionRead_ChemCam_Data.triggered.connect(
                lambda: self.addWidget(core.ReadChemCamData.ReadChemCamData))
            self.actionRemove_Baseline.triggered.connect(
                lambda: self.addWidget(core.BaselineRemoval.BaselineRemoval))
            self.actionCross_Validation.triggered.connect(
                lambda: self.addWidget(core.CrossValidation.CrossValidation))
            self.actionDimensionality_Reduction.triggered.connect(
                lambda: self.addWidget(core.DimensionalityReduction.DimensionalityReduction))
            self.actionInterpolate.triggered.connect(
                lambda: self.addWidget(core.Interpolation.Interpolation))
            self.actionLoad_Data.triggered.connect(
                lambda: self.addWidget(core.LoadData.LoadData))
            self.actionSave_Current_Data.triggered.connect(
                lambda: self.addWidget(core.WriteToCSV.WriteToCSV))
            self.actionRename_Data.triggered.connect(
                lambda: self.addWidget(core.RenameData.RenameData))
            self.actionApply_Mask.triggered.connect(
                lambda: self.addWidget(core.MaskData.MaskData))
            self.actionMultiply_by_Vector.triggered.connect(
                lambda: self.addWidget(core.MultiplyByVector.MultiplyByVector))
            self.actionNormalization.triggered.connect(
                lambda: self.addWidget(core.Normalization.Normalization))
            self.actionSet_Output_Path.triggered.connect(
                lambda: self.addWidget(core.OutputFolder.OutputFolder))
            self.actionPeak_Areas.triggered.connect(
                lambda: self.addWidget(core.PeakAreas.PeakAreas))
            self.actionPlot.triggered.connect(
                lambda: self.addWidget(core.Plot.Plot))
            self.actionPlot_ICA_PCA.triggered.connect(
                lambda: self.addWidget(core.Plot_ICA_PCA.Plot_ICA_PCA))
            self.actionPlot_Spectra.triggered.connect(
                lambda: self.addWidget(core.PlotSpectra.PlotSpectra))
            self.actionTrain.triggered.connect(
                lambda: self.addWidget(core.RegressionTrain.RegressionTrain))
            self.actionPredict.triggered.connect(
                lambda: self.addWidget(core.RegressionPredict.RegressionPredict))
            self.actionRemove_Rows.triggered.connect(
                lambda: self.addWidget(core.RemoveRows.RemoveRows))
            self.actionSplit_Data.triggered.connect(
                lambda: self.addWidget(core.SplitDataset.SplitDataset))
            self.actionOutlier_Removal.triggered.connect(
                lambda: self.addWidget(core.OutlierRemoval.OutlierRemoval))
            self.actionStratified_Folds.triggered.connect(
                lambda: self.addWidget(core.StratifiedFolds.StratifiedFolds))
            self.actionSubmodel_Predict.triggered.connect(
                lambda: self.addWidget(core.SubmodelPredict.SubmodelPredict))
            self.actionSpectral_Derivative.triggered.connect(
                lambda: self.addWidget(core.SpecDeriv.SpecDeriv))
            self.actionCombine_Data_Sets.triggered.connect(
                lambda: self.addWidget(core.CombineDataSets.CombineDataSets))
            self.actionData_Box.triggered.connect(self.on_DataTable_clicked)
            self.actionAbout.triggered.connect(self.on_About_clicked)
            self.actionQtmodern.triggered.connect(lambda: self.theme('qtmodern'))
            self.actionDefault.triggered.connect(lambda: self.theme('default'))
            self.actionBrace_yourself.triggered.connect(lambda: self.theme('braceyourself'))
            self.actionCreate_New_Workflow.triggered.connect(self.new)
            self.actionClear_Workflow.triggered.connect(self.clear)
            self.actionSave_Current_Workflow.triggered.connect(self.on_save_clicked)
            self.actionRestore_Workflow.triggered.connect(self.on_restore_clicked)
            self.deleteModulePushButton.clicked.connect(self.on_delete_module_clicked)
            self.okPushButton.clicked.connect(self.on_okButton_clicked)
            self.undoModulePushButton.clicked.connect(self.on_Rerun_Button_clicked)
            self.stopPushButton.clicked.connect(self.on_stopButton_clicked)
            self.actionOn.triggered.connect(self.debug_mode)
            self.actionOff.triggered.connect(self.normal_mode)
            self.actionExit.triggered.connect(self.MainWindow.close)

        except Exception as e:
            print(e)

    def closeEvent(self, event):
        """
        Write window size and position to config file
        """
        self._writeWindowAttributeSettings()
        event.accept()

    def getWidgetItems(self):
        """
        This function iterates through widgetList
        gets the name of all the fuctions
        and then all of the parameters in the UI
        and write it to a list to be returned

        Iterate through widgetList
        get the names of core, add it to a temp l
        add f list to be the first item in ui_items
        """
        f = []
        ui_items = []
        for f_items in self.widgetList:
            f.append(type(f_items).__name__)
        ui_items.append(f)

        for dat in self.widgetList:
            ui_items.append(dat.getGuiParams())
        return ui_items

    def setWidgetItems(self, dict):
        """
        This function iterates through a `dict`
        and restores the UI

        :param dict:
        :return:
        """
        for f_items in dict[0]:
            """
            Really complex way of running essentially this:
            `self.addWidget(core.SplitDataset.SplitDataset))`
            Part of the reason why we're doing this is because we're saving function
            names to a list, you can't save function instances. So this is the next
            best thing.
            """
            self.addWidget(getattr(getattr(core, f_items), f_items))

        for i in range(1, len(dict)):
            self.widgetList[i - 1].setGuiParams(dict[i])

    def on_save_clicked(self):
        """
        Save the workflow to a *.wrf file

        :return:
        """
        try:
            filename, _filter = QtWidgets.QFileDialog.getSaveFileName(None,
                                                                      "Choose where you want save your file",
                                                                      self.outpath,
                                                                      '(*.wrf)')
            print(filename)
            with open(filename, 'wb') as fp:
                pickle.dump(self.getWidgetItems(), fp)
            self.title.setFileName(filename.split('/')[-1])
            self.MainWindow.setWindowTitle(self.title.display())
        except Exception as e:
            print("File not loaded {}".format(e))

    def on_restore_clicked(self):
        """
        Choose a file
        Set the file to be read
        Pickle load our data and push it through the function setWidgetItems

        :return:
        """
        try:
            self.restorefilename, _filter = QtWidgets.QFileDialog.getOpenFileName(None,
                                                                                  "Open Workflow File",
                                                                                  self.outpath,
                                                                                  '(*.wrf)')
            print(self.restorefilename)
            with open(self.restorefilename, 'rb') as fp:
                self.setWidgetItems(pickle.load(fp))
            self.title.setFileName(self.restorefilename.split('/')[-1])
            self.MainWindow.setWindowTitle(self.title.display())
        except Exception as e:
            print("File not loaded: {}".format(e))

    def on_delete_module_clicked(self):
        """
        Check to see if the last item is enabled
        If it is, delete the last item in the list
        And then call the del_layout function, which will remove the item from the UI

        :return:
        """
        try:
            if self.widgetList[-1].isEnabled():
                del self.widgetList[-1]
                delete.del_layout(self.verticalLayout_3)
            else:
                print("Cannot delete")
        except:
            print("Cannot delete")

    def on_okButton_clicked(self):
        """
        Start the multithreading function

        :return:
        """
        self.onStart()
        self.taskFinished.connect(self.onFinished)

    def on_Rerun_Button_clicked(self):
        """
        Check to see if the last item in the list is enabled
        subtract 1 from leftOff
        enable the current leftOff item

        :return:
        """
        try:
            if not self.widgetList[-1].isEnabled():
                self.leftOff -= 1
                self.widgetList[self.leftOff].setDisabled(False)
        except:
            pass

    def on_stopButton_clicked(self):
        """
        Terminate running thread

        :return:
        """
        if self.isRunning():
            self.terminate()
            self.taskFinished.emit()
        else:
            print("There is nothing running right now")

    def on_About_clicked(self):
        self.aboutForm = core.About.About()
        self.aboutForm.show()

    def on_DataTable_clicked(self):
        self.dataForm = core.DataTable.DataTable()
        self.dataForm.show()

    def _writeWindowAttributeSettings(self):
        '''
        Save window attributes as settings.

        Called when window moved, resized, or closed.
        '''
        self.settings.beginGroup("mainWindow")
        self.settings.setValue("pos", self.MainWindow.pos())
        self.settings.setValue("maximized", self.MainWindow.isMaximized())
        if not self.MainWindow.isMaximized():
            self.settings.setValue("size", self.MainWindow.size())

        self.settings.endGroup()

    def _readAndApplyWindowAttributeSettings(self):
        '''
        Read window attributes from settings,
        using current attributes as defaults (if settings not exist.)

        Called at QMainWindow initialization, before show().
        '''
        self.settings.beginGroup("mainWindow")
        # No need for toPoint, etc. : PySide converts types
        try:
            self.MainWindow.move(self.settings.value("pos"))
            if self.settings.value("maximized") in 'true':
                self.MainWindow.showMaximized()
            else:
                self.MainWindow.resize(self.settings.value("size"))
        except:
            pass
        self.settings.endGroup()

    def onStart(self):
        """
        This is multithreading thus run() == start()
        make the bar pulse green
        TaskThread.start()

        :return:
        """
        self.progressBar.setRange(0, 0)
        self.start()

    def onFinished(self):  # onFinished function
        self.progressBar.setRange(0, 1)  # stop the bar pulsing green
        self.progressBar.setValue(1)  # displays 100% after process is finished.

    def clear(self):
        """
        Delete all modules in GUI

        :return:
        """
        while len(self.widgetList) > 0 and self.widgetList[-1].isEnabled():
            self.on_delete_module_clicked()
        self.title.setFileName('')
        self.MainWindow.setWindowTitle(self.title.display())
        self.textBrowser.clear()

    def new(self):
        """
        Start a new gui

        :return:
        """
        self._writeWindowAttributeSettings()
        p = mp.Process(target=main, args=())
        p.start()

    def runModules(self):
        """
        This function iterates through a list of object addresses
        which then run it's dot notated function()

        iterate through our widgets, start from the last left off item
        get the name of our current widget item
        start the timers
        print the name of the module running
        if a restored file exists
            run connectWidgets # to update the current UI widget
            run selectiveRestore # to select the right items
            Terminate running process, and let the user decide if they want to continue forward
        run our current modules function()
        get our end time
        print how long it took our current module to execute based on start time and end time
        disable our current module
        increment our left off module

        :return:
        """
        dic = None
        try:
            with open(self.restorefilename, 'rb') as fp:
                dic = pickle.load(fp)
        except:
            pass

        for modules in range(self.leftOff, len(self.widgetList)):
            name_ = type(self.widgetList[modules]).__name__
            s = time.time()
            print("{} Module is Running...".format(name_))
            # if dic is not None:
            #     self.widgetList[modules].connectWidgets()
            #     self.widgetList[modules].selectiveSetGuiParams(dic[modules + 1])
            self.widgetList[modules].function()
            e = time.time()
            print("Module {} executed in: {} seconds".format(name_, e - s))
            self.widgetList[modules].setDisabled(True)
            self.leftOff = modules + 1

    def _exceptionLogger(self, function):
        """
        Logs an exception that occurs during the running of a function
        Take a function address in as an input

        :param function:
        :return:
        """
        logfile = "file.log"
        logpath = os.path.join(os.path.join(os.environ['USERPROFILE']), 'Desktop\\logs')

        try:
            function()
        except:
            if not os.path.exists(logpath):
                os.makedirs(logpath)
            timenow = strftime('%d-%m-%y_%H-%M-%S')
            logfilename = "%s_%s" % (timenow, logfile)
            filename = os.path.join(str(os.getcwd()), "%s" % (os.path.join(logpath, logfilename)))
            logging.basicConfig(level=logging.DEBUG, filename=filename)
            logging.exception(
                '[%s %s] (%s) Version: %s:' % (platform.system(), platform.release(), timenow, __version__))
            traceback.print_exc()
            print('\nException was logged to "%s"' % (os.path.join(logpath, logfilename)))

    def run(self):
        """
        Start the thread for running all the modules

        :return:
        """
        if self.debug:
            self._exceptionLogger(self.runModules)
        else:
            try:
                self.runModules()
            except Exception as e:
                print("Your module broke: please fix.", e)
                try:
                    self.widgetList[self.leftOff].setDisabled(False)
                except:
                    pass
        self.taskFinished.emit()


def get_splash(app):
    """
    Get the splash screen for the application
    But check to see if the image even exists

    :param app:
    :return:
    """
    dirs = ['../images/', '/point_spectra_gui/images', './point_spectra_gui/images']
    for dir in dirs:
        if os.path.exists(dir + 'splash.png'):
            splash_pix = QPixmap(dir + 'splash.png')  # default
            app_icon = QtGui.QIcon(dir + 'icon.png')
            app.setWindowIcon(app_icon)
            splash = QSplashScreen(splash_pix, Qt.WindowStaysOnTopHint)
            splash.setMask(splash_pix.mask())
            splash.show()
            time.sleep(0.5)
            app.processEvents()
            return 0


def setDarkmode(app):
    settings = QSettings('USGS', 'PPSG')
    p = settings.value('theme') == 'qtmodern'
    if q and p:
        qtmodern.styles.dark(app)


def main():
    sys._excepthook = sys.excepthook
    sys.excepthook = my_exception_hook
    app = QtWidgets.QApplication(sys.argv)
    get_splash(app)
    setDarkmode(app)
    mainWindow = QtWidgets.QMainWindow()
    ui = MainWindow()
    ui.setupUi(mainWindow)
    mainWindow.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



from PyQt5 import QtWidgets
from sklearn.linear_model.omp import OrthogonalMatchingPursuit
from sklearn.linear_model.omp import OrthogonalMatchingPursuitCV

from point_spectra_gui.ui.OMP import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class Ui_Form(Ui_Form, OrthogonalMatchingPursuit, OrthogonalMatchingPursuitCV, Basics):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.formGroupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        self.fitInterceptCheckBox.setChecked(self.fit_intercept)
        self.normalizeCheckBox.setChecked(self.normalize)
        self.defaultComboItem(self.precomputeComboBox, self.precompute)

    def function(self):
        params = {'fit_intercept': self.fitInterceptCheckBox.isChecked(),
                  'normalize': self.normalizeCheckBox.isChecked(),
                  'precompute': self.precomputeComboBox.currentText(),
                  'CV': self.cVCheckBox.isChecked()}

        return params, self.getChangedValues(params, OrthogonalMatchingPursuit())


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formGroupBox = QtWidgets.QGroupBox(Form)
        self.formGroupBox.setObjectName("formGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.formGroupBox)
        self.formLayout.setObjectName("formLayout")
        self.dataSet1Label = QtWidgets.QLabel(self.formGroupBox)
        self.dataSet1Label.setObjectName("dataSet1Label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.dataSet1Label)
        self.dataSet1ComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.dataSet1ComboBox.setObjectName("dataSet1ComboBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.dataSet1ComboBox)
        self.dataSet2Label = QtWidgets.QLabel(self.formGroupBox)
        self.dataSet2Label.setObjectName("dataSet2Label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.dataSet2Label)
        self.dataSet2ComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.dataSet2ComboBox.setObjectName("dataSet2ComboBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.dataSet2ComboBox)
        self.outputToDataSetLabel = QtWidgets.QLabel(self.formGroupBox)
        self.outputToDataSetLabel.setObjectName("outputToDataSetLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.outputToDataSetLabel)
        self.outputToDataSetComboBox = QtWidgets.QComboBox(self.formGroupBox)
        self.outputToDataSetComboBox.setObjectName("outputToDataSetComboBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.outputToDataSetComboBox)
        self.verticalLayout.addWidget(self.formGroupBox)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.formGroupBox.setTitle(("Combine Data Sets"))
        self.dataSet1Label.setText(("Data Set 1"))
        self.dataSet2Label.setText(("Data Set 2"))
        self.outputToDataSetLabel.setText(("Output to DataSet: "))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




import ast

from PyQt5 import QtWidgets,QtCore
from sklearn.linear_model.ridge import Ridge
from sklearn.linear_model.ridge import RidgeCV

from point_spectra_gui.ui.cv_Ridge import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class Ui_Form(Ui_Form, Ridge, RidgeCV, Basics):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.Ridge

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        ridge = Ridge()

        self.alphaLineEdit.setText('0.01, 0.1, 1.0, 10, 100')
        self.fit_intercept_list.setCurrentItem(self.fit_intercept_list.findItems(str(ridge.fit_intercept),QtCore.Qt.MatchExactly)[0])
        self.normalize_list.setCurrentItem(self.normalize_list.findItems(str(ridge.normalize),QtCore.Qt.MatchExactly)[0])
        self.toleranceLineEdit.setText(str(ridge.tol))
        self.maxNumOfIterationslineEdit.setText(str(ridge.max_iter))


    def function(self):
        fit_intercept_items = [i.text() == 'True' for i in self.fit_intercept_list.selectedItems()]
        normalize_items = [i.text() == 'True' for i in self.normalize_list.selectedItems()]
        try:
            max_iter = [int(i) for i in self.maxNumOfIterationslineEdit.text().split(',')]
        except:
            max_iter = [None]
        params = {'alpha': [float(i) for i in self.alphaLineEdit.text().split(',')],
                  'copy_X': [True],
                  'fit_intercept': fit_intercept_items,
                  'max_iter': max_iter,
                  'normalize': normalize_items,
                  'solver': ['auto'],
                  'tol': [float(i) for i in self.toleranceLineEdit.text().split(',')],
                  'random_state': [None]}
        modelkey = str(params)
        return params, modelkey


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets

from Qtickle import Qtickle
from point_spectra_gui.core.dimensionalityReductionMethods import *
from point_spectra_gui.ui.DimensionalityReduction import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class DimensionalityReduction(Ui_Form, Basics):
    def setupUi(self, Form):
        self.Form = Form
        super().setupUi(Form)
        Basics.setupUi(self, Form)
        self.dimensionalityReductionMethods()

    def get_widget(self):
        return self.formGroupBox

    def connectWidgets(self):
        self.algorithm_list = ['Choose an algorithm',
                               'PCA',
                               'FastICA',
                               'JADE-ICA',
                               't-SNE',
                               'LLE']

        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.setComboBox(self.chooseMethodComboBox, self.algorithm_list)
        self.chooseMethodComboBox.currentIndexChanged.connect(
            lambda: self.make_dimred_widget(self.chooseMethodComboBox.currentText()))

    def getGuiParams(self):
        """
        Overriding Basics' getGuiParams, because I'll need to do a list of lists
        in order to obtain the regressionMethods' parameters
        """
        self.qt = Qtickle.Qtickle(self)
        s = []
        s.append(self.qt.guiSave())
        for items in self.alg:
            s.append(items.getGuiParams())
        return s

    def setGuiParams(self, dict):
        self.qt = Qtickle.Qtickle(self)
        self.qt.guiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].setGuiParams(dict[i])

    def selectiveSetGuiParams(self, dict):
        """
        Override Basics' selective Restore function

        Setup Qtickle
        selectively restore the UI, the data to do that will be in the 0th element of the dictionary
        We will then iterate through the rest of the dictionary
        Will now restore the parameters for the algorithms in the list, Each of the algs have their own selectiveSetGuiParams

        :param dict:
        :return:
        """

        self.qt = Qtickle.Qtickle(self)
        self.qt.selectiveGuiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].selectiveSetGuiParams(dict[i])

    def function(self):
        method = self.chooseMethodComboBox.currentText()
        datakey = self.chooseDataComboBox.currentText()
        # xvars = [str(x.text()) for x in self.xVariableList.selectedItems()]
        params, modelkey = self.getMethodParams(self.chooseMethodComboBox.currentIndex())
        load_fit = False
        col = 'wvl'
        self.data[datakey].dim_red(col, method, [], params, load_fit=load_fit)

    def make_dimred_widget(self, alg, params=None):
        self.hideAll()
        print(alg)
        for i in range(len(self.algorithm_list)):
            if alg == self.algorithm_list[i]:
                self.alg[i - 1].setHidden(False)

    def hideAll(self):
        for a in self.alg:
            a.setHidden(True)

    def dimensionalityReductionMethods(self):
        self.alg = []
        list_forms = [dimred_PCA,
                      dimred_FastICA,
                      dimred_JADE,
                      dimred_tSNE,
                      dimred_LLE]
        for items in list_forms:
            self.alg.append(items.Ui_Form())
            self.alg[-1].setupUi(self.Form)
            self.dim_reduction_vlayout.addWidget(self.alg[-1].get_widget())
            self.alg[-1].setHidden(True)

    def getMethodParams(self, index):
        return self.alg[index - 1].function()


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)

    Form = QtWidgets.QWidget()
    ui = DimensionalityReduction()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



import sys
import warnings

from point_spectra_gui import core

try:
    import qtmodern.styles

    q = True
except:
    q = False
    warnings.warn("You're missing the qtmodern package."
                  "to install it use pip install qtmodern")

from PyQt5 import QtWidgets

from point_spectra_gui.core.MainWindow import MainWindow


class Ui_MainWindow(MainWindow):
    def connectWidgets(self):
        """
        Connect all the widgets associated with the MainWindow UI
        :return:
        """
        self.addWidget(core.ReadChemCamData.ReadChemCamData)
        self.addWidget(core.BaselineRemoval.BaselineRemoval)
        self.addWidget(core.CrossValidation.CrossValidation)
        self.addWidget(core.DimensionalityReduction.DimensionalityReduction)
        self.addWidget(core.Interpolation.Interpolation)
        self.addWidget(core.LoadData.LoadData)
        self.addWidget(core.WriteToCSV.WriteToCSV)
        self.addWidget(core.RenameData.RenameData)
        self.addWidget(core.MaskData.MaskData)
        self.addWidget(core.MultiplyByVector.MultiplyByVector)
        self.addWidget(core.Normalization.Normalization)
        self.addWidget(core.OutputFolder.OutputFolder)
        self.addWidget(core.PeakAreas.PeakAreas)
        self.addWidget(core.Plot.Plot)
        self.addWidget(core.Plot_ICA_PCA.Plot_ICA_PCA)
        self.addWidget(core.PlotSpectra.PlotSpectra)
        self.addWidget(core.RegressionTrain.RegressionTrain)
        self.addWidget(core.RegressionPredict.RegressionPredict)
        self.addWidget(core.RemoveRows.RemoveRows)
        self.addWidget(core.SplitDataset.SplitDataset)
        self.addWidget(core.StratifiedFolds.StratifiedFolds)
        self.addWidget(core.SubmodelPredict.SubmodelPredict)


def main():
    app = QtWidgets.QApplication(sys.argv)
    mainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(mainWindow)
    mainWindow.show()
    app.exec_()


if __name__ == '__main__':
    main()



import inspect
from distutils.util import strtobool

from PyQt5 import QtCore
from PyQt5.QtWidgets import *


class Qtickle(object):
    """
    A script designed to aid in saving values inside Qt objects.
    It works by inspecting for objectnames and then collecting their
    values.

    The name is a portmanteau of Qt and pickle. Originally the script
    utilized pickle to save data. Now data is returned in the form of
    a dictionary. This way the user gets to choose what happens to the
    data
    """

    def __init__(self, ui):
        self.ui = ui

    def guiSave(self):
        """
        Save all values in a particular UI
        :return:
        """
        dict = {}
        # Save geometry
        # self.settings.setValue('size', self.core.size())
        # self.settings.setValue('pos', self.core.pos())
        try:
            for name, obj in inspect.getmembers(self.ui):
                if isinstance(obj, QLineEdit):
                    name = obj.objectName()
                    value = obj.text()
                    dict[name] = value

                if isinstance(obj, QCheckBox):
                    name = obj.objectName()
                    state = obj.isChecked()
                    dict[name] = state

                if isinstance(obj, QRadioButton):
                    name = obj.objectName()
                    value = obj.isChecked()  # get stored value from registry
                    dict[name] = value

                if isinstance(obj, QSpinBox):
                    name = obj.objectName()
                    value = obj.value()  # get stored value from registry
                    dict[name] = value

                if isinstance(obj, QDoubleSpinBox):
                    name = obj.objectName()
                    value = obj.value()
                    dict[name] = value

                if isinstance(obj, QSlider):
                    name = obj.objectName()
                    value = obj.value()  # get stored value from registry
                    dict[name] = value

                if isinstance(obj, QLabel):
                    name = obj.objectName()
                    value = obj.text()
                    dict[name] = value

                if isinstance(obj, QComboBox):
                    values = []  # the list that will hold all values from QCombobox
                    name = obj.objectName()  # get the QCombobox object's name
                    for i in range(obj.count()):  # QCombobox contains a number of items
                        itemData = obj.itemText(i)
                        values.append(itemData)  # put those items into a list for saving
                    index = obj.findText(obj.currentText())  # return the index of the item, assign to selected
                    dict[name + "_values"] = values  # save all the values in settings
                    dict[name + "_index"] = index  # save the indexed value in settings

                if isinstance(obj, QListWidget):
                    values = []
                    name = obj.objectName()
                    for i in range(obj.count()):
                        itemData = obj.item(i).text()
                        values.append(itemData)
                    dict[name + "_values"] = values
                    # since there is a possibility of multiple items,
                    # we'll just save the string representation of those items to be restored
                    dict[name + "_index"] = [str(x.text()) for x in obj.selectedItems()]

            return dict
        except Exception as e:
            print(e)

    def guiRestore(self, dict):
        # Restore geometry
        # self.core.resize(self.settings.value('size', QtCore.QSize(500, 500)))
        # self.core.move(self.settings.value('pos', QtCore.QPoint(60, 60)))
        for name, obj in inspect.getmembers(self.ui):
            pass
            try:
                if isinstance(obj, QLineEdit):
                    name = obj.objectName()
                    value = dict[name]
                    obj.setText(value)  # restore lineEditFile

                if isinstance(obj, QCheckBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        try:
                            obj.setChecked(strtobool(value))  # restore checkbox
                        except:
                            obj.setChecked(value)

                if isinstance(obj, QRadioButton):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setChecked(strtobool(value))

                if isinstance(obj, QSlider):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QDoubleSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(float(value))

                if isinstance(obj, QLabel):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setText(value)

                if isinstance(obj, QComboBox):
                    name = obj.objectName()
                    values = dict[name + "_values"]
                    # clear all the objects
                    # so that we don't run into issues
                    # with restoring the list of values
                    obj.clear()
                    if values is not None:
                        for value in values:
                            if not (value == '' or value == "") and len(value) > 0:
                                # if there are some values in the list, we should add them to the Combobox
                                obj.addItem(value)

                    index = dict[name + "_index"]  # next we want to select the item in question by getting it's index
                    obj.setCurrentIndex(int(index))

                if isinstance(obj, QListWidget):
                    name = obj.objectName()
                    values = dict[name + "_values"]
                    index = dict[name + "_index"]
                    obj.clear()
                    if values is not None:
                        for value in values:
                            list_item = QListWidgetItem(value)
                            obj.addItem(list_item)
                        for i in index:
                            matching_items = obj.findItems(i, QtCore.Qt.MatchExactly)
                            for item in matching_items:
                                obj.setCurrentItem(item)

            except Exception as e:
                print(e)

    def isGuiChanged(self, functionCall):
        try:
            for name, obj in inspect.getmembers(self.ui):
                if isinstance(obj, QLineEdit):
                    obj.textChanged.connect(lambda: functionCall())

                if isinstance(obj, QCheckBox):
                    obj.stateChanged.connect(lambda: functionCall())

                if isinstance(obj, QRadioButton):
                    obj.toggled.connect(lambda: functionCall())

                if isinstance(obj, QSpinBox):
                    obj.valueChanged.connect(lambda: functionCall())

                if isinstance(obj, QDoubleSpinBox):
                    obj.valueChanged.connect(lambda: functionCall())

                if isinstance(obj, QSlider):
                    obj.event.connect(lambda: functionCall())

                if isinstance(obj, QComboBox):
                    obj.currentIndexChanged.connect(lambda: functionCall())

                    # if isinstance(obj, QListWidget): This needs to be added at somepoint
                    #     obj.

        except Exception as e:
            print(e)

    def selectiveGuiRestore(self, dict):
        for name, obj in inspect.getmembers(self.ui):
            pass
            try:
                if isinstance(obj, QCheckBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        try:
                            obj.setChecked(strtobool(value))  # restore checkbox
                        except:
                            obj.setChecked(value)

                if isinstance(obj, QRadioButton):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setChecked(strtobool(value))

                if isinstance(obj, QSlider):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(int(value))  # restore value from registry

                if isinstance(obj, QDoubleSpinBox):
                    name = obj.objectName()
                    value = dict[name]
                    if value is not None:
                        obj.setValue(float(value))

                if isinstance(obj, QComboBox):
                    name = obj.objectName()
                    # select the item in question by setting it's index
                    index = dict[name + "_index"]
                    obj.setCurrentIndex(int(index))

                if isinstance(obj, QListWidget):
                    name = obj.objectName()
                    values = dict[name + "_values"]
                    index = dict[name + "_index"]
                    obj.clear()
                    if values is not None:
                        for value in values:
                            list_item = QListWidgetItem(value)
                            obj.addItem(list_item)
                        for i in index:
                            matching_items = obj.findItems(i, QtCore.Qt.MatchExactly)
                            for item in matching_items:
                                obj.setCurrentItem(item)

            except Exception as e:
                print(e)



import numpy as np
import pandas as pd
from PyQt5 import QtWidgets
from libpysat.regression import cv
from libpysat.spectral.spectral_data import spectral_data

from Qtickle import Qtickle
from point_spectra_gui.core.crossValidateMethods import *
from point_spectra_gui.ui.CrossValidation import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class CrossValidation(Ui_Form, Basics):
    def setupUi(self, Form):
        self.Form = Form
        super().setupUi(Form)
        Basics.setupUi(self, Form)
        self.regressionMethods()

    def get_widget(self):
        return self.groupLayout

    def make_regression_widget(self, alg, params=None):
        self.hideAll()
        print(alg)
        for i in range(len(self.algorithm_list) - 1):
            if alg == self.algorithm_list[i] and i > 0:
                self.alg[i - 1].setHidden(False)

    def connectWidgets(self):
        self.algorithm_list = ['Choose an algorithm',
                               'ARD',
                               'BRR',
                               'Elastic Net',
                               'GP',
                               # 'KRR',  This needs more work since it requires parameters for the kernel passed as an object
                               'LARS',
                               'LASSO',
                               'LASSO LARS',
                               'OLS',
                               'OMP',
                               'PLS',
                               'Ridge',
                               'SVR']

        self.setComboBox(self.chooseDataComboBox, self.datakeys)
        self.setComboBox(self.chooseAlgorithmComboBox, self.algorithm_list)
        self.yMaxDoubleSpinBox.setMaximum(999999)
        self.yMinDoubleSpinBox.setMaximum(999999)
        self.yMaxDoubleSpinBox.setValue(100)
        self.changeComboListVars(self.yVariableList, self.yvar_choices())
        self.changeComboListVars(self.xVariableList, self.xvar_choices())
        self.xvar_choices()
        self.chooseAlgorithmComboBox.currentIndexChanged.connect(
            lambda: self.make_regression_widget(self.chooseAlgorithmComboBox.currentText()))
        self.chooseDataComboBox.currentIndexChanged.connect(
            lambda: self.changeComboListVars(self.yVariableList, self.yvar_choices()))
        self.chooseDataComboBox.currentIndexChanged.connect(
            lambda: self.changeComboListVars(self.xVariableList, self.xvar_choices()))

    def getGuiParams(self):
        """
        Overriding Basics' getGuiParams, because I'll need to do a list of lists
        in order to obtain the regressionMethods' parameters
        """
        self.qt = Qtickle.Qtickle(self)
        s = []
        s.append(self.qt.guiSave())
        for items in self.alg:
            s.append(items.getGuiParams())
        return s

    def setGuiParams(self, dict):
        self.qt = Qtickle.Qtickle(self)
        self.qt.guiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].setGuiParams(dict[i])

    def selectiveSetGuiParams(self, dict):
        """
        Override Basics' selective Restore function

        Setup Qtickle
        selectively restore the UI, the data to do that will be in the 0th element of the dictionary
        We will then iterate through the rest of the dictionary
        Will now restore the parameters for the algorithms in the list, Each of the algs have their own selectiveSetGuiParams

        :param dict:
        :return:
        """

        self.qt = Qtickle.Qtickle(self)
        self.qt.selectiveGuiRestore(dict[0])
        for i in range(len(dict)):
            self.alg[i - 1].selectiveSetGuiParams(dict[i])

    def function(self):
        method = self.chooseAlgorithmComboBox.currentText()
        datakey = self.chooseDataComboBox.currentText()
        xvars = [str(x.text()) for x in self.xVariableList.selectedItems()]
        yvars = [('comp', str(y.text())) for y in self.yVariableList.selectedItems()]
        yrange = [self.yMinDoubleSpinBox.value(), self.yMaxDoubleSpinBox.value()]
        params, modelkey = self.getMethodParams(self.chooseAlgorithmComboBox.currentIndex())

        y = np.array(self.data[datakey].df[yvars])
        match = np.squeeze((y > yrange[0]) & (y < yrange[1]))
        data_for_cv = spectral_data(self.data[datakey].df.ix[match])
        # Warning: Params passing through cv.cv(params) needs to be in lists
        # Example: {'n_components': [4], 'scale': [False]}
        cv_obj = cv.cv(params)
        self.data[datakey].df, self.cv_results, cvmodels, cvmodelkeys = cv_obj.do_cv(data_for_cv.df, xcols=xvars,
                                                                                     ycol=yvars,
                                                                                     yrange=yrange, method=method)
        for n, key in enumerate(cvmodelkeys):
            self.modelkeys.append(key)
            self.models[key] = cvmodels[n]
            self.model_xvars[key] = xvars
            self.model_yvars[key] = yvars
            if method != 'GP':
                coef = np.squeeze(cvmodels[n].model.coef_)
                coef = pd.DataFrame(coef)
                coef.index = pd.MultiIndex.from_tuples(self.data[datakey].df[xvars].columns.values)
                coef = coef.T
                coef[('meta', 'Model')] = key
                try:
                    coef[('meta', 'Intercept')] = cvmodels[n].model.intercept_
                except:
                    pass
                try:
                    self.data['Model Coefficients'] = spectral_data(
                        pd.concat([self.data['Model Coefficients'].df, coef]))
                except:
                    self.data['Model Coefficients'] = spectral_data(coef)
                    self.datakeys.append('Model Coefficients')

        self.datakeys.append('CV Results ' + modelkey)
        self.data['CV Results ' + modelkey] = self.cv_results

    def yvar_choices(self):
        try:
            yvarchoices = self.data[self.chooseDataComboBox.currentText()].df['comp'].columns.values
            yvarchoices = [i for i in yvarchoices if not 'Unnamed' in i]  # remove unnamed columns from choices
        except:
            yvarchoices = ['No composition columns!']
        return yvarchoices

    def xvar_choices(self):
        try:
            xvarchoices = self.data[self.chooseDataComboBox.currentText()].df.columns.levels[0].values
            xvarchoices = [i for i in xvarchoices if not 'Unnamed' in i]  # remove unnamed columns from choices
        except:
            xvarchoices = ['No valid choices!']
        return xvarchoices

    def hideAll(self):
        for a in self.alg:
            a.setHidden(True)

    def getMethodParams(self, index):
        return self.alg[index - 1].function()

    def regressionMethods(self):
        self.alg = []
        list_forms = [cv_ARD,
                      cv_BayesianRidge,
                      cv_ElasticNet,
                      cv_GP,
                      #              cv_KRR,
                      cv_LARS,
                      cv_Lasso,
                      cv_LassoLARS,
                      cv_OLS,
                      cv_OMP,
                      cv_PLS,
                      cv_Ridge,
                      cv_SVR
                      ]
        for items in list_forms:
            self.alg.append(items.Ui_Form())
            self.alg[-1].setupUi(self.Form)
            self.algorithmLayout.addWidget(self.alg[-1].get_widget())
            self.alg[-1].setHidden(True)


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = CrossValidation()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.
import pandas as pd
from PyQt5 import QtWidgets

from point_spectra_gui.ui.CombineDataSets import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class CombineDataSets(Ui_Form, Basics):
    def setupUi(self, Form):
        self.Form = Form
        super().setupUi(Form)
        Basics.setupUi(self, Form)

    def get_widget(self):
        return self.formGroupBox

    def connectWidgets(self):
        self.setComboBox(self.dataSet1ComboBox, self.datakeys)
        self.setComboBox(self.dataSet2ComboBox, self.datakeys)
        self.setComboBox(self.outputToDataSetComboBox, self.datakeys)

    def function(self):
        dataSet1 = self.dataSet1ComboBox.currentText()
        dataSet2 = self.dataSet2ComboBox.currentText()
        dataIn = self.outputToDataSetComboBox.currentText()
        self.data[dataIn] = pd.concat([dataSet1, dataSet2])


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setObjectName("groupBox")
        self.ENLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.ENLayout.setObjectName("ENLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.elasticNetCVGroupBox = QtWidgets.QGroupBox(self.groupBox)
        self.elasticNetCVGroupBox.setObjectName("elasticNetCVGroupBox")
        self.ElasticNetCV_2 = QtWidgets.QFormLayout(self.elasticNetCVGroupBox)
        self.ElasticNetCV_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.ElasticNetCV_2.setObjectName("ElasticNetCV_2")
        self.l1_ratioLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.l1_ratioLabel.setObjectName("l1_ratioLabel")
        self.ElasticNetCV_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.l1_ratioLabel)
        self.l1_ratioDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.elasticNetCVGroupBox)
        self.l1_ratioDoubleSpinBox.setObjectName("l1_ratioDoubleSpinBox")
        self.ElasticNetCV_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.l1_ratioDoubleSpinBox)
        self.epsLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.epsLabel.setObjectName("epsLabel")
        self.ElasticNetCV_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.epsLabel)
        self.epsDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.elasticNetCVGroupBox)
        self.epsDoubleSpinBox.setObjectName("epsDoubleSpinBox")
        self.ElasticNetCV_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.epsDoubleSpinBox)
        self.n_alphasLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.n_alphasLabel.setObjectName("n_alphasLabel")
        self.ElasticNetCV_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.n_alphasLabel)
        self.n_alphasSpinBox = QtWidgets.QSpinBox(self.elasticNetCVGroupBox)
        self.n_alphasSpinBox.setObjectName("n_alphasSpinBox")
        self.ElasticNetCV_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.n_alphasSpinBox)
        self.alphasLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.alphasLabel.setObjectName("alphasLabel")
        self.ElasticNetCV_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.alphasLabel)
        self.alphasLineEdit = QtWidgets.QLineEdit(self.elasticNetCVGroupBox)
        self.alphasLineEdit.setObjectName("alphasLineEdit")
        self.ElasticNetCV_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.alphasLineEdit)
        self.fit_interceptLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.fit_interceptLabel.setObjectName("fit_interceptLabel")
        self.ElasticNetCV_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.fit_interceptLabel)
        self.fit_interceptCheckBox = QtWidgets.QCheckBox(self.elasticNetCVGroupBox)
        self.fit_interceptCheckBox.setObjectName("fit_interceptCheckBox")
        self.ElasticNetCV_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.fit_interceptCheckBox)
        self.normalizeLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.normalizeLabel.setObjectName("normalizeLabel")
        self.ElasticNetCV_2.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.normalizeLabel)
        self.normalizeCheckBox = QtWidgets.QCheckBox(self.elasticNetCVGroupBox)
        self.normalizeCheckBox.setObjectName("normalizeCheckBox")
        self.ElasticNetCV_2.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.normalizeCheckBox)
        self.precomputeLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.precomputeLabel.setObjectName("precomputeLabel")
        self.ElasticNetCV_2.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.precomputeLabel)
        self.precomputeComboBox = QtWidgets.QComboBox(self.elasticNetCVGroupBox)
        self.precomputeComboBox.setObjectName("precomputeComboBox")
        self.ElasticNetCV_2.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.precomputeComboBox)
        self.max_iterLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.max_iterLabel.setObjectName("max_iterLabel")
        self.ElasticNetCV_2.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.max_iterLabel)
        self.max_iterSpinBox = QtWidgets.QSpinBox(self.elasticNetCVGroupBox)
        self.max_iterSpinBox.setObjectName("max_iterSpinBox")
        self.ElasticNetCV_2.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.max_iterSpinBox)
        self.tolLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.tolLabel.setObjectName("tolLabel")
        self.ElasticNetCV_2.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.tolLabel)
        self.tolDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.elasticNetCVGroupBox)
        self.tolDoubleSpinBox.setObjectName("tolDoubleSpinBox")
        self.ElasticNetCV_2.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.tolDoubleSpinBox)
        self.copy_XLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.copy_XLabel.setObjectName("copy_XLabel")
        self.ElasticNetCV_2.setWidget(11, QtWidgets.QFormLayout.LabelRole, self.copy_XLabel)
        self.copy_XCheckBox = QtWidgets.QCheckBox(self.elasticNetCVGroupBox)
        self.copy_XCheckBox.setObjectName("copy_XCheckBox")
        self.ElasticNetCV_2.setWidget(11, QtWidgets.QFormLayout.FieldRole, self.copy_XCheckBox)
        self.verboseLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.verboseLabel.setObjectName("verboseLabel")
        self.ElasticNetCV_2.setWidget(12, QtWidgets.QFormLayout.LabelRole, self.verboseLabel)
        self.verboseCheckBox = QtWidgets.QCheckBox(self.elasticNetCVGroupBox)
        self.verboseCheckBox.setObjectName("verboseCheckBox")
        self.ElasticNetCV_2.setWidget(12, QtWidgets.QFormLayout.FieldRole, self.verboseCheckBox)
        self.n_jobsLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.n_jobsLabel.setObjectName("n_jobsLabel")
        self.ElasticNetCV_2.setWidget(13, QtWidgets.QFormLayout.LabelRole, self.n_jobsLabel)
        self.n_jobsSpinBox = QtWidgets.QSpinBox(self.elasticNetCVGroupBox)
        self.n_jobsSpinBox.setObjectName("n_jobsSpinBox")
        self.ElasticNetCV_2.setWidget(13, QtWidgets.QFormLayout.FieldRole, self.n_jobsSpinBox)
        self.positiveLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.positiveLabel.setObjectName("positiveLabel")
        self.ElasticNetCV_2.setWidget(14, QtWidgets.QFormLayout.LabelRole, self.positiveLabel)
        self.positiveCheckBox = QtWidgets.QCheckBox(self.elasticNetCVGroupBox)
        self.positiveCheckBox.setObjectName("positiveCheckBox")
        self.ElasticNetCV_2.setWidget(14, QtWidgets.QFormLayout.FieldRole, self.positiveCheckBox)
        self.selectionLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.selectionLabel.setObjectName("selectionLabel")
        self.ElasticNetCV_2.setWidget(15, QtWidgets.QFormLayout.LabelRole, self.selectionLabel)
        self.selectionComboBox = QtWidgets.QComboBox(self.elasticNetCVGroupBox)
        self.selectionComboBox.setObjectName("selectionComboBox")
        self.ElasticNetCV_2.setWidget(15, QtWidgets.QFormLayout.FieldRole, self.selectionComboBox)
        self.cVLabel = QtWidgets.QLabel(self.elasticNetCVGroupBox)
        self.cVLabel.setObjectName("cVLabel")
        self.ElasticNetCV_2.setWidget(9, QtWidgets.QFormLayout.LabelRole, self.cVLabel)
        self.cVSpinBox = QtWidgets.QSpinBox(self.elasticNetCVGroupBox)
        self.cVSpinBox.setObjectName("cVSpinBox")
        self.ElasticNetCV_2.setWidget(9, QtWidgets.QFormLayout.FieldRole, self.cVSpinBox)
        self.horizontalLayout.addWidget(self.elasticNetCVGroupBox)
        self.elasticNetGroupBox = QtWidgets.QGroupBox(self.groupBox)
        self.elasticNetGroupBox.setObjectName("elasticNetGroupBox")
        self.ElasticNet = QtWidgets.QFormLayout(self.elasticNetGroupBox)
        self.ElasticNet.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.ElasticNet.setObjectName("ElasticNet")
        self.enalphaLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enalphaLabel.setObjectName("enalphaLabel")
        self.ElasticNet.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.enalphaLabel)
        self.enalphaDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.elasticNetGroupBox)
        self.enalphaDoubleSpinBox.setObjectName("enalphaDoubleSpinBox")
        self.ElasticNet.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.enalphaDoubleSpinBox)
        self.enl1_ratioLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enl1_ratioLabel.setObjectName("enl1_ratioLabel")
        self.ElasticNet.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.enl1_ratioLabel)
        self.enl1_ratioDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.elasticNetGroupBox)
        self.enl1_ratioDoubleSpinBox.setObjectName("enl1_ratioDoubleSpinBox")
        self.ElasticNet.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.enl1_ratioDoubleSpinBox)
        self.enfit_interceptLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enfit_interceptLabel.setObjectName("enfit_interceptLabel")
        self.ElasticNet.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.enfit_interceptLabel)
        self.enfit_interceptCheckBox = QtWidgets.QCheckBox(self.elasticNetGroupBox)
        self.enfit_interceptCheckBox.setObjectName("enfit_interceptCheckBox")
        self.ElasticNet.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.enfit_interceptCheckBox)
        self.ennormalizeLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.ennormalizeLabel.setObjectName("ennormalizeLabel")
        self.ElasticNet.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.ennormalizeLabel)
        self.ennormalizeCheckBox = QtWidgets.QCheckBox(self.elasticNetGroupBox)
        self.ennormalizeCheckBox.setObjectName("ennormalizeCheckBox")
        self.ElasticNet.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.ennormalizeCheckBox)
        self.enprecomputeLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enprecomputeLabel.setObjectName("enprecomputeLabel")
        self.ElasticNet.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.enprecomputeLabel)
        self.enprecomputeCheckBox = QtWidgets.QCheckBox(self.elasticNetGroupBox)
        self.enprecomputeCheckBox.setObjectName("enprecomputeCheckBox")
        self.ElasticNet.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.enprecomputeCheckBox)
        self.enmax_iterLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enmax_iterLabel.setObjectName("enmax_iterLabel")
        self.ElasticNet.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.enmax_iterLabel)
        self.enmax_iterSpinBox = QtWidgets.QSpinBox(self.elasticNetGroupBox)
        self.enmax_iterSpinBox.setObjectName("enmax_iterSpinBox")
        self.ElasticNet.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.enmax_iterSpinBox)
        self.encopy_XLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.encopy_XLabel.setObjectName("encopy_XLabel")
        self.ElasticNet.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.encopy_XLabel)
        self.encopy_XCheckBox = QtWidgets.QCheckBox(self.elasticNetGroupBox)
        self.encopy_XCheckBox.setObjectName("encopy_XCheckBox")
        self.ElasticNet.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.encopy_XCheckBox)
        self.entolLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.entolLabel.setObjectName("entolLabel")
        self.ElasticNet.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.entolLabel)
        self.entolDoubleSpinBox = QtWidgets.QDoubleSpinBox(self.elasticNetGroupBox)
        self.entolDoubleSpinBox.setObjectName("entolDoubleSpinBox")
        self.ElasticNet.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.entolDoubleSpinBox)
        self.enwarm_startLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enwarm_startLabel.setObjectName("enwarm_startLabel")
        self.ElasticNet.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.enwarm_startLabel)
        self.enwarm_startCheckBox = QtWidgets.QCheckBox(self.elasticNetGroupBox)
        self.enwarm_startCheckBox.setObjectName("enwarm_startCheckBox")
        self.ElasticNet.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.enwarm_startCheckBox)
        self.enpositiveLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enpositiveLabel.setObjectName("enpositiveLabel")
        self.ElasticNet.setWidget(9, QtWidgets.QFormLayout.LabelRole, self.enpositiveLabel)
        self.enpositiveCheckBox = QtWidgets.QCheckBox(self.elasticNetGroupBox)
        self.enpositiveCheckBox.setObjectName("enpositiveCheckBox")
        self.ElasticNet.setWidget(9, QtWidgets.QFormLayout.FieldRole, self.enpositiveCheckBox)
        self.enselectionLabel = QtWidgets.QLabel(self.elasticNetGroupBox)
        self.enselectionLabel.setObjectName("enselectionLabel")
        self.ElasticNet.setWidget(10, QtWidgets.QFormLayout.LabelRole, self.enselectionLabel)
        self.enselectionComboBox = QtWidgets.QComboBox(self.elasticNetGroupBox)
        self.enselectionComboBox.setObjectName("enselectionComboBox")
        self.enselectionComboBox.addItem("")
        self.enselectionComboBox.addItem("")
        self.ElasticNet.setWidget(10, QtWidgets.QFormLayout.FieldRole, self.enselectionComboBox)
        self.horizontalLayout.addWidget(self.elasticNetGroupBox)
        self.ENLayout.addLayout(self.horizontalLayout)
        self.CVCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.CVCheckBox.setObjectName("CVCheckBox")
        self.ENLayout.addWidget(self.CVCheckBox)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(Form)
        self.CVCheckBox.toggled['bool'].connect(self.elasticNetCVGroupBox.setVisible)
        self.CVCheckBox.toggled['bool'].connect(self.elasticNetGroupBox.setHidden)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(("Form"))
        self.elasticNetCVGroupBox.setTitle(("Elastic Net CV"))
        self.l1_ratioLabel.setText(_translate("Form", "l1_ratio\n"
""))
        self.l1_ratioDoubleSpinBox.setToolTip(_translate("Form", "The ElasticNet mixing parameter, with 0 <= l1_ratio <= 1. For l1_ratio\n"
"= 0 the penalty is an L2 penalty. For l1_ratio = 1 it is an L1 penalty.\n"
"For 0 < l1_ratio < 1, the penalty is a combination of L1 and L2."))
        self.l1_ratioDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.epsLabel.setText(_translate("Form", "eps\n"
""))
        self.epsDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.n_alphasLabel.setText(_translate("Form", "n_alphas\n"
""))
        self.n_alphasSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.alphasLabel.setText(_translate("Form", "alphas\n"
""))
        self.alphasLineEdit.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.fit_interceptLabel.setText(_translate("Form", "fit_intercept\n"
""))
        self.fit_interceptCheckBox.setToolTip(_translate("Form", "Whether the intercept should be estimated or not. If False, the data\n"
"is assumed to be already centered."))
        self.fit_interceptCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.normalizeLabel.setText(_translate("Form", "normalize\n"
""))
        self.normalizeCheckBox.setToolTip(_translate("Form", "This parameter is ignored when fit_intercept is set to False. If True,\n"
"the regressors X will be normalized before regression by subtracting\n"
"the mean and dividing by the l2-norm. If you wish to standardize,\n"
"please use sklearn.preprocessing. StandardScaler before calling fit on\n"
"an estimator with normalize=False."))
        self.normalizeCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.precomputeLabel.setText(_translate("Form", "precompute\n"
""))
        self.precomputeComboBox.setToolTip(_translate("Form", "Whether to use a precomputed Gram matrix to speed up calculations.\n"
"The Gram matrix can also be passed as argument. For sparse input this\n"
"option is always True to preserve sparsity."))
        self.precomputeComboBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.max_iterLabel.setText(_translate("Form", "max_iter\n"
""))
        self.max_iterSpinBox.setToolTip(("The maximum number of iterations"))
        self.max_iterSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.tolLabel.setText(_translate("Form", "tol\n"
""))
        self.tolDoubleSpinBox.setToolTip(("The tolerance for the optimization:"))
        self.tolDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.copy_XLabel.setText(_translate("Form", "copy_X\n"
""))
        self.copy_XCheckBox.setToolTip(("If True, X will be copied; else, it may be overwritten."))
        self.copy_XCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.verboseLabel.setText(_translate("Form", "verbose\n"
""))
        self.verboseCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.n_jobsLabel.setText(_translate("Form", "n_jobs\n"
""))
        self.n_jobsSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.positiveLabel.setText(_translate("Form", "positive\n"
""))
        self.positiveCheckBox.setToolTip(("When set to True, forces the coefficients to be positive."))
        self.positiveCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.selectionLabel.setText(("selection"))
        self.selectionComboBox.setToolTip(_translate("Form", "If set to \'random\', a random coefficient is updated every iteration\n"
"rather than looping over features sequentially by default. This\n"
"(setting to \'random\') often leads to significantly faster convergence\n"
"especially when tol is higher than 1e-4."))
        self.selectionComboBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.cVLabel.setText(("cv"))
        self.cVSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.elasticNetGroupBox.setTitle(("Elastic Net"))
        self.enalphaLabel.setText(_translate("Form", "alpha\n"
""))
        self.enalphaDoubleSpinBox.setToolTip(_translate("Form", "Constant that multiplies the penalty terms. Defaults to 1.0. See the\n"
"notes for the exact mathematical meaning of this parameter. alpha = 0\n"
"is equivalent to an ordinary least square, solved by the\n"
"LinearRegression object. For numerical reasons, using alpha = 0 with\n"
"the Lasso object is not advised. Given this, you should use the\n"
"LinearRegression object."))
        self.enalphaDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enl1_ratioLabel.setText(_translate("Form", "l1_ratio\n"
""))
        self.enl1_ratioDoubleSpinBox.setToolTip(_translate("Form", "The ElasticNet mixing parameter, with 0 <= l1_ratio <= 1. For l1_ratio\n"
"= 0 the penalty is an L2 penalty. For l1_ratio = 1 it is an L1 penalty.\n"
"For 0 < l1_ratio < 1, the penalty is a combination of L1 and L2."))
        self.enl1_ratioDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enfit_interceptLabel.setText(_translate("Form", "fit_intercept\n"
""))
        self.enfit_interceptCheckBox.setToolTip(_translate("Form", "Whether the intercept should be estimated or not. If False, the data\n"
"is assumed to be already centered."))
        self.enfit_interceptCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.ennormalizeLabel.setText(_translate("Form", "normalize\n"
""))
        self.ennormalizeCheckBox.setToolTip(_translate("Form", "This parameter is ignored when fit_intercept is set to False. If True,\n"
"the regressors X will be normalized before regression by subtracting\n"
"the mean and dividing by the l2-norm. If you wish to standardize,\n"
"please use sklearn.preprocessing. StandardScaler before calling fit on\n"
"an estimator with normalize=False."))
        self.ennormalizeCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enprecomputeLabel.setText(_translate("Form", "precompute\n"
""))
        self.enprecomputeCheckBox.setToolTip(_translate("Form", "Whether to use a precomputed Gram matrix to speed up calculations.\n"
"The Gram matrix can also be passed as argument. For sparse input this\n"
"option is always True to preserve sparsity."))
        self.enprecomputeCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enmax_iterLabel.setText(_translate("Form", "max_iter\n"
""))
        self.enmax_iterSpinBox.setToolTip(("The maximum number of iterations"))
        self.enmax_iterSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.encopy_XLabel.setText(_translate("Form", "copy_X\n"
""))
        self.encopy_XCheckBox.setToolTip(("If True, X will be copied; else, it may be overwritten."))
        self.encopy_XCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.entolLabel.setText(_translate("Form", "tol\n"
""))
        self.entolDoubleSpinBox.setToolTip(("The tolerance for the optimization:"))
        self.entolDoubleSpinBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enwarm_startLabel.setText(_translate("Form", "warm_start\n"
""))
        self.enwarm_startCheckBox.setToolTip(_translate("Form", "When set to True, reuse the solution of the previous call to fit as\n"
"initialization, otherwise, just erase the previous solution."))
        self.enwarm_startCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enpositiveLabel.setText(("positive"))
        self.enpositiveCheckBox.setToolTip(("When set to True, forces the coefficients to be positive."))
        self.enpositiveCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enselectionLabel.setText(("selection"))
        self.enselectionComboBox.setToolTip(_translate("Form", "If set to \'random\', a random coefficient is updated every iteration\n"
"rather than looping over features sequentially by default. This\n"
"(setting to \'random\') often leads to significantly faster convergence\n"
"especially when tol is higher than 1e-4."))
        self.enselectionComboBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.enselectionComboBox.setItemText(0, ("cyclic"))
        self.enselectionComboBox.setItemText(1, ("random"))
        self.CVCheckBox.setToolTip(("Cross Validate"))
        self.CVCheckBox.setWhatsThis(("http://scikit-learn.org/stable/modules/generated/sklearn.linear_model.ElasticNet.html"))
        self.CVCheckBox.setText(("CV"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




from PyQt5 import QtWidgets, QtCore
from sklearn.linear_model.least_angle import LassoLars, LassoLarsCV, LassoLarsIC
import numpy as np
from point_spectra_gui.ui.cv_LassoLARS import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class Ui_Form(Ui_Form, Basics):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        self.connectWidgets()

    def get_widget(self):
        return self.formGroupBox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        # LassoLARS
        ll = LassoLars()
        self.minalpha_spin.setValue(0.0000001)
        self.maxalpha_spin.setValue(0.01)
        self.nalpha_spin.setValue(100)
        self.fit_intercept_list.setCurrentItem(self.fit_intercept_list.findItems(str(ll.fit_intercept),QtCore.Qt.MatchExactly)[0])
        self.normalize_list.setCurrentItem(self.normalize_list.findItems(str(ll.normalize),QtCore.Qt.MatchExactly)[0])
        self.max_iterLineEdit.setText(str(ll.max_iter))
        self.force_positive_list.setCurrentItem(self.force_positive_list.findItems(str(ll.positive),QtCore.Qt.MatchExactly)[0])


    def function(self):
        fit_intercept_items = [i.text() == 'True' for i in self.fit_intercept_list.selectedItems()]
        normalize_items = [i.text() == 'True' for i in self.normalize_list.selectedItems()]
        positive_items = [i.text() == 'True' for i in self.force_positive_list.selectedItems()]
        alphas = np.logspace(np.log10(self.minalpha_spin.value() * 1e10), np.log10(self.maxalpha_spin.value() * 1e10),
                             num=self.nalpha_spin.value())
        params = {
            'alpha': alphas,
            'fit_intercept': fit_intercept_items,
            'verbose': [True],
            'normalize': normalize_items,
            'precompute': ['auto'],
            'max_iter': [int(i) for i in self.max_iterLineEdit.text().split(',')],
            'copy_X': [True],
            'fit_path': [False],
            'positive': positive_items
        }


        modelkey = str(params)
        return params, modelkey


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets
from libpysat.spectral.baseline_code.fabc import FABC

from point_spectra_gui.ui.FABC import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class Ui_Form(Ui_Form, Basics):
    def setupUi(self, Form):
        super().setupUi(Form)
        self.checkMinAndMax()
        Basics.setupUi(self, Form)

    def get_widget(self):
        return self.groupbox

    def setHidden(self, bool):
        self.get_widget().setHidden(bool)

    def connectWidgets(self):
        br = FABC()
        self.smoothnessDoubleSpinBox.setValue(br.dilation_)
        self.dilationSpinBox.setValue(br.smoothness_)

    def function(self):
        methodParameters = {'dilation_': self.smoothnessDoubleSpinBox.value(),
                            'smoothness_': self.dilationSpinBox.value()}
        return methodParameters, self.getChangedValues(methodParameters, FABC())


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())



import numpy as np
from PyQt5 import QtWidgets

from point_spectra_gui.ui.Normalization import Ui_Form
from point_spectra_gui.util.BasicFunctionality import Basics


class norm_range:
    def __init__(self, minLabel, minSpin, maxLabel, maxSpin, hidden):
        self.minLabel = minLabel
        self.minSpin = minSpin
        self.maxLabel = maxLabel
        self.maxSpin = maxSpin
        self.hidden = hidden
        self.hidden.setHidden(True)
        self.hidden.stateChanged.connect(self.Hidden)
        self.setLimits(0.0, 9999999.0)

    def Hidden(self):
        if self.hidden.isChecked():
            self.minLabel.setHidden(True)
            self.minSpin.setHidden(True)
            self.maxLabel.setHidden(True)
            self.maxSpin.setHidden(True)
        else:
            self.minLabel.setHidden(False)
            self.minSpin.setHidden(False)
            self.maxLabel.setHidden(False)
            self.maxSpin.setHidden(False)

    def getValues(self):
        return self.minSpin.value(), self.maxSpin.value()

    def setLimits(self, min, max):
        self.minSpin.setMinimum(min)
        self.maxSpin.setMinimum(min)
        self.minSpin.setMaximum(max)
        self.maxSpin.setMaximum(max)

    def setValue(self, int_):
        self.minSpin.setValue(int_)
        self.maxSpin.setValue(int_)


class Normalization(Ui_Form, Basics):
    def setupUi(self, Form, restore=False):
        if restore:
            self.restored = True
        super().setupUi(Form)
        self.setup_norm_ranges()
        self.index.setHidden(True)
        Basics.setupUi(self, Form)

    def get_widget(self):
        return self.groupBox

    def setup_norm_ranges(self):
        self.ranges = [norm_range(self.minWvlLabel, self.minWvlSpin, self.maxWvlLabel, self.maxWvlSpin,
                                  self.hidden_1),
                       norm_range(self.minWvlLabel_2, self.minWvlSpin_2, self.maxWvlLabel_2, self.maxWvlSpin_2,
                                  self.hidden_2),
                       norm_range(self.minWvlLabel_3, self.minWvlSpin_3, self.maxWvlLabel_3, self.maxWvlSpin_3,
                                  self.hidden_3),
                       norm_range(self.minWvlLabel_4, self.minWvlSpin_4, self.maxWvlLabel_4, self.maxWvlSpin_4,
                                  self.hidden_4),
                       norm_range(self.minWvlLabel_5, self.minWvlSpin_5, self.maxWvlLabel_5, self.maxWvlSpin_5,
                                  self.hidden_5),
                       norm_range(self.minWvlLabel_6, self.minWvlSpin_6, self.maxWvlLabel_6, self.maxWvlSpin_6,
                                  self.hidden_6),
                       norm_range(self.minWvlLabel_7, self.minWvlSpin_7, self.maxWvlLabel_7, self.maxWvlSpin_7,
                                  self.hidden_7),
                       norm_range(self.minWvlLabel_8, self.minWvlSpin_8, self.maxWvlLabel_8, self.maxWvlSpin_8,
                                  self.hidden_8),
                       norm_range(self.minWvlLabel_9, self.minWvlSpin_9, self.maxWvlLabel_9, self.maxWvlSpin_9,
                                  self.hidden_9),
                       norm_range(self.minWvlLabel_10, self.minWvlSpin_10, self.maxWvlLabel_10, self.maxWvlSpin_10,
                                  self.hidden_10),
                       norm_range(self.minWvlLabel_11, self.minWvlSpin_11, self.maxWvlLabel_11, self.maxWvlSpin_11,
                                  self.hidden_11),
                       norm_range(self.minWvlLabel_12, self.minWvlSpin_12, self.maxWvlLabel_12, self.maxWvlSpin_12,
                                  self.hidden_12),
                       norm_range(self.minWvlLabel_13, self.minWvlSpin_13, self.maxWvlLabel_13, self.maxWvlSpin_13,
                                  self.hidden_13),
                       norm_range(self.minWvlLabel_14, self.minWvlSpin_14, self.maxWvlLabel_14, self.maxWvlSpin_14,
                                  self.hidden_14),
                       norm_range(self.minWvlLabel_15, self.minWvlSpin_15, self.maxWvlLabel_15, self.maxWvlSpin_15,
                                  self.hidden_15)]
        self.spin_list = []
        for i in self.ranges:
            self.spin_list.append(i.minSpin)
            self.spin_list.append(i.maxSpin)
            i.hidden.setChecked(True)
        self.ranges[0].hidden.setChecked(False)

    def setDataLimits(self):
        # if the data exists, get the min and max of the data and use as the limits for the ranges
        datamin = 0.0
        datamax = 9999999.0
        try:
            data = self.data[self.chooseDataComboBox.currentText()]
            tempmin = np.min(data.df[self.varToNormalizeListWidget.currentItem().text()].columns.values)
            tempmax = np.max(data.df[self.varToNormalizeListWidget.currentItem().text()].columns.values)
            if type(tempmin) is not str:
                datamin = tempmin
            if type(tempmax) is not str:
                datamax = tempmax
        except:
            pass

        for i in range(len(self.ranges)):
            self.ranges[i].setLimits(datamin, datamax)
        try:
            if type(tempmin) is not str:
                self.setDataRanges()
        except:
            pass

    def setDataRanges(self):
        x = self.data[self.chooseDataComboBox.currentText()].df[
            self.varToNormalizeListWidget.currentItem().text()].columns.values
        # borrowed this from baseline removal code, which in turn borrowed from matplotlib's boxplot outlier detection.
        d = np.diff(x)
        q1, q3 = np.percentile(d, (25, 75))
        cutoff = q3 + 1.5 * (q3 - q1)
        inds, = np.where(d > cutoff)
        xvals = (x[inds] + x[inds + 1]) / 2.0  # set values to the middle of the gaps
        xvals = np.append(xvals, np.max(x))
        self.index.setValue(len(xvals))
        print('index set to ' + str(self.index.value()))
        for i, val in enumerate(xvals):
            self.ranges[i].maxSpin.setValue(val)
            self.ranges[i].hidden.setChecked(False)

    def updateVal(self):
        # step through the spinboxes and make sure that each successive box value is
        # greater than or equal to the one before it
        for i in range(1, len(self.spin_list)):
            if self.spin_list[i].value() < self.spin_list[i - 1].value():
                self.spin_list[i].setValue(self.spin_list[i - 1].value())

    def connectWidgets(self):
        # populate combobox with dataset names
        self.setComboBox(self.chooseDataComboBox, self.datakeys)

        # update list of variables to normalize when data set selection is changed
        self.chooseDataComboBox.currentIndexChanged.connect(
            lambda: self.changeComboListVars(self.varToNormalizeListWidget, self.xvar_choices()))
        self.changeComboListVars(self.varToNormalizeListWidget, self.xvar_choices())

        # when the variable to normalize is changed, update the min max and default norm ranges
        self.varToNormalizeListWidget.itemSelectionChanged.connect(self.setDataLimits)

        # when anything in the gui is changed, run update val to make sure values are increasing
        self.qt.isGuiChanged(self.updateVal)

        # connect the add and delete ranges buttons
        self.add_range_button.clicked.connect(lambda: self.on_addRange_pushed())
        self.delete_range_button.clicked.connect(lambda: self.on_deleteRange_pushed())

    def on_addRange_pushed(self):
        if self.index.value() < len(self.ranges):
            self.ranges[self.index.value()].hidden.setChecked(False)
            self.index.setValue(self.index.value() + 1)
            print('Index is now: ' + str(self.index.value()))
        else:
            print("Cannot add more ranges!")

    def on_deleteRange_pushed(self):
        if self.index.value() > 1:
            self.index.setValue(self.index.value() - 1)
            self.ranges[self.index.value()].hidden.setChecked(True)
            print('Index is now: ' + str(self.index.value()))
        else:
            print("Cannot delete any more ranges!")

    def function(self):
        datakey = self.chooseDataComboBox.currentText()
        range_vals = []
        for i in self.ranges:
            range_min, range_max = i.getValues()
            if range_min != range_max:
                range_vals.append([range_min, range_max])
                pass
        try:
            col_var = self.varToNormalizeListWidget.currentItem().text()
        except:
            print("Did you remember to select a variable?")
        print("{}".format(range_vals))
        try:
            self.data[datakey].norm(range_vals, col_var)
            print("Normalization has been applied to the ranges: " + str(range_vals))
        except Exception as e:
            print("There was a problem: ", e)

    def xvar_choices(self):
        try:
            try:
                xvarchoices = self.data[self.chooseDataComboBox.currentText()].df.columns.levels[
                    0].values
            except:
                xvarchoices = self.data[self.chooseDataComboBox.currentText()].columns.values
            xvarchoices = [i for i in xvarchoices if not 'Unnamed' in i]  # remove unnamed columns from choices
        except:
            xvarchoices = ['No valid choices!']
        return xvarchoices


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)

    Form = QtWidgets.QWidget()
    ui = Normalization()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())
